<datatables>
    <style id="datatables-riot-style">
        .datatablesMouseOverRow {
            background-color: #fffacd !important;
        }
        .dataTables_filter {
            margin-top: 30px;
        }
        .dataTables_info {
            display: contents;
            line-height: 50px;
        }
        .dataTables_info:before {
            content: " ";
            margin-left: 15px;
        }
        .dataTables_length {
            margin-top: 10px;
        }
        .pagination li a {
            padding: 4px 10px;
        }
        #btn-new-devices  {
            margin-top: 28px;
            margin-bottom: 12px;
            margin-right: 15px;
        }
        #btn-new-devices > .dt-buttons > .buttons-create {
            border-radius: 5px;
        }
        #success-message, #error-message {
            margin-top: 28px;
            margin-left: 15px;
            padding: 5px;
        }
        .button-csv-import {
            float: inherit;
            margin-right: 15px !important;
        }
    </style>
    <div class="page-title">
        <h1>{props.title}</h1>
        <div id="error-message" class="alert alert-danger pull-left hide"></div>
        <div id="success-message" class="alert alert-success pull-left hide"></div>
        <div id="btn-group" class="btn-group btn-gr pull-right"></div>
        <div id="btn-new-devices" class="pull-right" each="{button in props.leftButtons}">
            <div class="dt-buttons"><a class="dt-button buttons-create btn btn-nc" onclick="{button.onclick}"><span>{button.label}</span></a></div>
        </div>
        <div id="search-menu" class="pull-right"/>
        <div class="clearfix"></div>
    </div>
    <div class="panel-body">
        <div class="table-responsive col-lg-12 col-xs-12">
            <table id="{props.name}_datatables" class="table table-striped table-bordered table-hover" role="grid" width="100%" cellspacing="0">
                <thead>
                <th class="select-checkbox all-checkbox" if="{props.showAllCheckbox}"></th>
                <th if="{!props.showAllCheckbox}"></th>
                <th each="{item in props.items}">{item}</th>
                </thead>
                <tbody></tbody>
            </table>
        </div>
    </div>
    <loading/>
    <script>
        export default {
            onBeforeMount(props, state) {
                this.service = props.service
                this.scrutiny = props.scrutiny
                this.logoutModal = props.logoutModal
                this.registTrackimoDevice = props.registTrackimoDevice
                $('#error-message').empty().addClass('hide')
                $('#success-message').empty().addClass('hide')
            },
            onMounted(props, state) {
                const self = this
                $.getJSON('assets/json/datatables/' + props.name + '.json').then(async function(json, status) {
                    Function.prototype.toJSON = Function.prototype.toString
                    const parser = function(k,v){return v.toString().indexOf('function') === 0 ? eval('('+v+')') : v}
                    const jsonText = JSON.stringify(json)
                    let data = JSON.parse(jsonText)
                    // const data = JSON.parse(jsonText, parser)

                    if (props.addJSON) {
                        data = await props.addJSON(data)
                    }

                    self.buildDataTables(data, props.service, props)
                })
            },
            buildDataTables(data, service, props) {
                const self = this
                const editor = new $.fn.dataTable.Editor( {
                    table: '#' + props.name + '_datatables',
                    ajax: function ( method, url, input, successCallback, errorCallback ) {
                        const output = { data: [] }
                        const process = async function(options) {

                            if (input.action == 'create') {
                                const key = Object.keys(input.data)[0]
                                const data = input.data[key]
                                if (data.id == '') {
                                    data.id = 0
                                }
                                const result = await service.create(data)
                                if (result.status && result.status == "OK") {
                                    data.id = result.data ? result.data.id : 0
                                    data.caseNo = result.data ? result.data.caseNo : 0
                                    data.createdAt = result.data ? result.data.createdAt : ''
                                    data.rowId = Math.random()
                                    output.data.push(data)
                                    successCallback(output)
                                } else {
                                    errorCallback(output, result, result.message)
                                }
                            } else if (input.action == 'edit') {

                                const key = Object.keys(input.data)[0]
                                const data = input.data[key]
                                // data.forEach(function (value, key) {
                                //     if (_.isArray(value)) {
                                //         const concatValue = ''
                                //         value.forEach(function (v) {
                                //             concatValue += v + ','
                                //         })
                                //         concatValue = concatValue.substring(0, concatValue.lastIndexOf(','))
                                //         data[key] = concatValue
                                //     }
                                // })
                                const result = await service.update(data)
                                if (result.status && result.status == "OK") {
                                    data.rowId = key
                                    output.data.push(data)
                                    successCallback(output)
                                } else {
                                    errorCallback(output, result, result.message)
                                }

                            } else if (input.action == 'remove') {
                                const param = self.buildRemoveParam(props, input)
                                const result = await service.delete(param)
                                if (result.status && result.status == "OK") {
                                    successCallback(output)
                                } else {
                                    errorCallback(output, result, result.message)
                                }
                            }
                        }

                        process(props)

                    },
                    idSrc:  'rowId',
                    i18n: {
                        // ボタンとモーダル
                        create: {
                            button: '新規',
                            title: '新規',
                            submit: '送信'
                        },
                        edit: {
                            button: '編集',
                            title: '編集',
                            submit: '送信'
                        },
                        remove: {
                            button: '削除',
                            title: '削除',
                            submit: '送信',
                            confirm: {
                                _: "%d行削除しますが、よろしいですか？",
                                1: "1行削除しますが、よろしいですか？"
                            }
                        },
                        datetime: {
                            previous: '前月',
                            next:     '翌月',
                            months:   [ '1月', '2月', '3月', '4月', '5月', '6月', '7月', '8月', '9月', '10月', '11月', '12月' ],
                            weekdays: [ '日', '月', '火', '水', '木', '金', '土' ],
                            amPm:     [ '午前', '午後' ],
                            unknown:  '-'
                        }
                    },
                    fields: data.fields
                } )

                const dataTable = $('#' + props.name + '_datatables').DataTable({
                    editor: editor,
                    dom: props.dom ? props.dom : 'flpitpi',
                    ajax: function (data, callback, settings) {

                        $(".pagination").hide()
                        $(".dataTables_filter").hide()
                        $(".input-sm").hide()
                        $("#btn-group").hide()

                        if (props.ajax) {

                            props.ajax(function(json) {
                                callback({
                                    data:json
                                })
                            })
                            return
                        }

                        service.list(data).done(response => {
                            if (response.status == "OK") {
                                const json = {
                                    data: response.data
                                }
                                callback(json)
                            } else {
                                toastr.error(response.message)
                            }
                        })
                    },
                    lengthMenu: props.lengthMenu ? props.lengthMenu : [ 5, 10, 25, 50, 100 ],
                    displayLength: props.displayLength ? props.displayLength : 10,
                    initComplete: function (datatable, data) {
                        $(".page-title").prepend($(".dataTables_filter"))
                        $(".dataTables_filter label").insertAfter(".dataTables_length").css({
                            "float": "right",
                            "margin-top": "7px",
                            "margin-right": "15px"
                        })

                        $('#' + props.name + '_datatables_wrapper .input-group:not(:last)').remove()
                        if (props.initComplete) return props.initComplete(datatable, data)
                    },
                    columnDefs: [
                        {targets:0, width: 15}
                    ],
                    columns: data.columns,
                    select: {
                        style: 'os',
                        selector: 'td:first-child'
                    },
                    rowCallback: function (row, data, rowNo) {
                        if (props.rowCallback) return props.rowCallback(row, data, rowNo)
                    },
                    order:props.order,
                    stateSave: false,
                    processing: true,
                    loadingRecords : true,
                    scrollCollapse: true,
                    scrollX: true,
                    destroy: true,
                    retrieve: true,
                    sorting: props.sorting ? props.sorting : [[1, 'asc']],
                    autoWidth: false,
                    fixedColumns: props.fixedColumns ? props.fixedColumns : {leftColumns: 2},
                    language: {
                        lengthMenu: '表示件数 _MENU_',
                        search: '<div class="input-group"><span class="input-group-addon"><span class="glyphicon glyphicon-search"></span></span>',
                        searchPlaceholder: '検索文字列を入力',
                        select: {
                            rows: {
                                _: "%d件選択中",
                                0: '',
                                1: "1件選択中"
                            }
                        },
                        zeroRecords: '検索結果は0件',
                        info: ' _TOTAL_ 件中 _START_ から _END_ まで表示',
                        infoFiltered: '全_MAX_件から検索',
                        infoEmpty:'',
                        paginate: {
                            sFirst:    '最初へ',
                            sPrevious: '前',
                            sNext:     '次',
                            sLast:     '最後'
                        }
                    }
                })

                this.dataTable = dataTable
                if (props.buttons && props.buttonsEvent) {
                    props.buttons()
                    props.buttonsEvent()
                } else if (props.buttons) {
                    props.buttons(dataTable, editor)
                } else {
                    // ボタン
                    let actions = [props.createAction ? _.extend({extend: "create", editor: editor, className: "btn btn-nc"}, {action: props.createAction}) : {extend: "create", editor: editor, className: "btn btn-nc"},
                        props.editAction ? _.extend({extend: "edit", editor: editor, className: "btn btn-nc"}, {action: props.editAction}) : {extend: "edit", editor: editor, className: "btn btn-nc"},
                        {extend: "remove", editor: editor, className: "btn btn-nc"}]

                    if (props.csvImport) {
                        actions.push({
                            text: 'Amz取込',
                            className: "dt-button btn btn-nc buttons-create button-csv-import",
                            action: function () {
                                uploadEditor.create( {
                                    title: 'Amz取込'
                                } );
                            }
                        })
                    }
                    new $.fn.dataTable.Buttons(dataTable, actions)
                    dataTable.buttons().container().addClass("insertSpace").appendTo($('.btn-group'))

                    const parent = $('<div/>', {class: 'dt-buttons'})
                    actions = $('<actioncontent/>')
                    parent.append(actions)
                    $('admincontent div#btn-group').append(parent)

                    // アクションボタン
                    const action = riot.mount('actioncontent', {
                        dt:dataTable,
                        editor: editor,
                        name: props.name,
                        actionButtons: props.actionButtons,
                        actionButtonsEvent: props.actionButtonsEvent,
                        adjustSize: props.adjustSize,
                        defaultMenu: true,
                        logoutModal: props.logoutModal,
                        scrutiny: props.scrutiny,
                    }, 'action')[0]
                }

                if (props.addOperation) props.addOperation()

                let selector = 'tbody tr td:not(:first-child,:nth-child(2))'
                if (props.disabledColumns && Array.isArray(props.disabledColumns)) {
                    const child = []
                    _.each(props.disabledColumns, function (columnNo) {
                        child.push(':nth-child(' + columnNo + ')')
                    })
                    selector = 'tbody tr td:not(:first-child,' + child.join(",") + ")"
                }

                // 編集可能な列
                $('#' + props.name + '_datatables').on( 'click', selector, function (e) {
                    editor.inline( this, {
                        submit: 'allIfChanged'
                    })
                })

                // データテーブルイベント
                dataTable.on('draw', function () {
                    $(".pagination").show()
                    $(".dataTables_filter").show()
                    $(".input-sm").show()

                    $("#btn-group").show()
                    if (props.draw) props.draw()
                })
                dataTable.on('select', function (e, dt, type, indexes) {
                    if (props.select) props.select(e, dt, type, indexes)
                })
                dataTable.on('deselect', function (e, dt, type, indexes) {
                    if (props.deselect) props.deselect(e, dt, type, indexes)
                })
                dataTable.on( 'buttons-action', function ( e, buttonApi, dataTable, node, config ) {
                    if (props.buttonsAction) props.buttonsAction(e, buttonApi, dataTable, node, config)
                } )

                // 編集のイベント
                editor
                    .on('preOpen', function (e, json, action) {
                        if (props.preOpen) return props.preOpen(e, json, action, this)
                    })
                    .on ('open',function (e, json, action) {
                        if (props.open) return props.open(e, json, action, this)
                    })
                    .on ('preSubmit', function (e, json, data) {
                        if (props.preSubmit) return props.preSubmit(e, json, data, this)
                    })
                    .on ('submitSuccess', function (e, json, data) {
                        if (props.submitSuccess) return props.submitSuccess(e, json, data, this)
                    })
                    .on('submitError', function (e, data, error) {
                        if (props.submitError) {
                            props.submitError(e, data, error)
                        }
                        this.error(error.errorMessage)
                    })

                // 非表示・読取専用項目
                if (editor.field('id') && props.name != "user") {
                    editor.disable('id')
                }

                // 鉛筆アイコン追加、削除
                $('#' + props.name + '_datatables').on('mouseenter', selector, function (e) {
                    $(e.currentTarget).append('<span class="fa fa-pencil fa-1"></span>')
                })
                $('#' + props.name + '_datatables').on('mouseout', selector, function (e) {
                    $(e.currentTarget).find('span.fa').remove()
                })
                // レイアウト
                $('.dataTables_filter ').addClass('pull-left input-group')
                $('.dataTables_filter input').addClass("search_type form-control")
                $('.dataTables_filter input').css('margin-left','0px')
                $('.dataTables_filter input').css('width','350px')

                // textareaの対応
                $('#' + props.name + '_datatables').on("keydown", "textarea", function(e) {
                    // enter key
                    if(e.keyCode == 13) {
                        if (e.shiftKey) {
                            return
                        }
                        e.preventDefault()
                        editor.submit()
                    }
                })
                // select2の対応
                $('#' + props.name + '_datatables').on("change", "select.select, input.select", function(e) {
                    const multiple = $(e.target).attr('data-multiple')
                    if (!multiple) {
                        const fieldName = e.currentTarget.id.replace('DTE_Field_','')
                        const preventVal = editor.s.editData[fieldName][_.keys(editor.s.editData[fieldName])[0]]
                        const currentVal = $(e.currentTarget).val()
                        if (currentVal == preventVal || !currentVal && !preventVal) return
                        if (currentVal) {
                            editor.s.editData[fieldName][_.keys(editor.s.editData[fieldName])[0]] = currentVal
                        }
                        editor.submit()
                    }
                    $(e.target).removeAttr('data-multiple')
                })

                // リサイズしても列の幅を合わせる
                $(window).resize(function() {
                    dataTable.columns.adjust().draw()
                })

                $('.buttons-csv,.buttons-pdf').removeClass().addClass('dt-button buttons-create btn btn-nc')

                // 行の高さを設定
                const target = $('#' + props.name + '_datatables tbody tr')
                const height = target.css('height')
                target.css('height', height * 0.75)

                // 行マウスオーバー時の背景色
                $('#' + props.name + '_datatables').on("mouseover", "tbody tr", function (e) {
                    $(this).addClass("datatablesMouseOverRow")
                    const index = $(e.currentTarget).index()
                    $('.dataTables_scrollBody table tbody tr:eq(' + index +')').addClass("datatablesMouseOverRow")
                    $('div.DTFC_LeftBodyLiner table tbody tr:eq(' + index +')').addClass("datatablesMouseOverRow")
                })
                $('#' + props.name + '_datatables').on("mouseout", "tbody tr", function (e) {
                    $(this).removeClass("datatablesMouseOverRow")
                    const index = $(e.currentTarget).index()
                    $('.dataTables_scrollBody table tbody tr:eq(' + index +')').removeClass("datatablesMouseOverRow")
                    $('div.DTFC_LeftBodyLiner table tbody tr:eq(' + index +')').removeClass("datatablesMouseOverRow")
                })

                // 読み取り専用のタイトル（ヘッダー）の背景色を変更
                _.each(props.disabledColumns, function (index) {
                    $(dataTable.table().header()).find('tr th:eq(' + (index-1) + ')').addClass("datatables-disabled-header")
                })

                const uploadEditor = this.getUploadEditor(props, dataTable)
            },
            buildRemoveParam (props, input) {
                const values = Object.values(input.data)
                const params = {
                }
                const removeKeys = JSON.parse(localStorage.getItem(Const.LOCAL_STORAGE_KEY_REMOVE_KEY))
                const itemName = removeKeys[props.name]

                const list = []
                if (Array.isArray(itemName)) {
                    values.forEach(value => {
                        const param = {}
                        itemName.forEach(item => {
                            param[item] = value[item]
                        })
                        list.push(param)
                    })
                    params.targets = list
                } else {
                    values.forEach(value => {
                        list.push(value[itemName])
                    })
                    params[itemName] = list
                }

                return params
            },
            getUploadEditor(props, dataTable) {

                $.fn.dataTable.Editor.upload = function (editor, conf, files, progressCallback, completeCallback) {
                    const reader = new FileReader();
                    const ids = [];
                    const generalError = 'CSVアップロード中にエラーが発生しました';

                    // Clear any existing errors, as the new upload might not be in error
                    editor.error(conf.name, '');

                    progressCallback(conf, conf.fileReadText || "<i>Uploading file</i>");

                    reader.onload = async function (e) {
                        await props.csvUpload(e, dataTable, editor, conf, files, generalError, completeCallback, ids)

                        // $.ajax($.extend({}, ajax, {
                        //     type: 'post',
                        //     data: data,
                        //     dataType: 'json',
                        //     contentType: false,
                        //     processData: false,
                        //     success: function (json) {
                        //         editor.off(`preSubmit.DTE_Upload`);
                        //         editor._event('uploadXhrSuccess', [conf.name, json]);
                        //
                        //         if (json.fieldErrors && json.fieldErrors.length) {
                        //             const errors = json.fieldErrors;
                        //
                        //             for (const i = 0, ien = errors.length; i < ien; i++) {
                        //                 editor.error(errors[i].name, errors[i].status);
                        //             }
                        //         } else if (json.error) {
                        //             editor.error(json.error);
                        //         } else {
                        //             completeCallback.call(editor, ids);
                        //             editor.submit();
                        //
                        //             toastr.options.timeOut = 10000
                        //             if (json.success_message) {
                        //                 toastr.success(json.success_message)
                        //             }
                        //             if (json.error_message) {
                        //                 toastr.options.escapeHTML = true
                        //                 toastr.error(json.error_message.join("<br>"))
                        //             }
                        //             if (json.success_message && !json.error_message) {
                        //                 setInterval(() => {
                        //                     location.href = location.href
                        //                 }, 1000)
                        //             }
                        //             if (json.success_message && json.error_message) {
                        //                 setInterval(() => {
                        //                     location.href = location.href
                        //                 }, 10000)
                        //             }
                        //             // dataTable.ajax.reload()
                        //             conf._input.find('div.upload button').html(conf.uploadText)
                        //             toastr.options.timeOut = 1000
                        //         }
                        //     },
                        //     error: function (xhr) {
                        //         editor._event('uploadXhrError', [conf.name, xhr]);
                        //         editor.error(conf.name, generalError);
                        //     }
                        // }));
                    };

                    reader.readAsArrayBuffer(files[0]);
                };

                return new $.fn.dataTable.Editor({
                    fields: [{
                        label: `CSVファイル:`,
                        name: 'csv',
                        type: 'upload',
                        uploadText: "CSVファイルを選択",
                        dragDropText: "CSVファイルをドラッグ＆ドロップしてください",
                        ajax: Settings.getValue("apiRoot") + "/" + props.name + "/csv",
                        fileReadText: "アップロード中..."
                    }]
                });
            }
        }

    </script>
</datatables>
